function model=pls2mod(varargin)
%Calculates a PLS model of specified complexity using either the NIPALS or
%SIMPLS algorithms. Most of the options are gathered in the structure
%variable 'options'.
%I/O: model=pls2mod(X,Y, LV, options) where the default options can be
%retrieved by writing pls2mod('options')

if nargin==1 &&strcmp(varargin{1}, 'options')
    model.prepr={'mean' 'mean'};
    model.algorithm='simpls';
    model.detailedoutput='on';
    return
elseif nargin==3
    
    X=varargin{1};
    Y=varargin{2};
    LV=varargin{3};
    opt.prepr={'mean' 'mean'};
    opt.algorithm='simpls';
    opt.detailedoutput='on';
    
elseif nargin==4
    X=varargin{1};
    Y=varargin{2};
    LV=varargin{3};
    opt=varargin{4};
end



[Xp, mx, sx]=prepfn(X, opt.prepr{1});    %Preprocessing of X matrix
[Yp, my, sy]=prepfn(Y, opt.prepr{2});    %Preprocessing of X matrix

[ns,nx]=size(X);

facts=min([LV nx ns-1]);

switch opt.algorithm
    case 'nipals'
        [T,P,U,R,W,Q,B, ssq]=pls2nip(Xp,Yp, facts);
        model.scores={T U};
        model.loadings={P Q};
        model.weights=W;
        model.Rweights=R;
        model.explvar=ssq;
        model.regcoef=B;
        [vip, viptot]=myvip2(T,W,R,Q,B);
        
    case 'simpls'
        [T,P,U,R,Q,B,V, ssq]=pls2sim(Xp,Yp, facts);
        model.scores={T U};
        model.loadings={P Q};
        model.Rweights=R;
        model.basis=V;
        model.explvar=ssq;
        model.regcoef=B;
        [vip, viptot]=myvip2(T,[],R,Q,B);
        
end

Yhat=Xp*B;
Ypred=unprepfn(Yhat,opt.prepr{2},my,sy);
model.preds=Ypred;
model.YResiduals=Y-Ypred;
model.bias=mean(model.YResiduals);
model.RMSE=sqrt(sum(model.YResiduals.^2)/size(Ypred,1));
RSS=sum(model.YResiduals.^2);
TSS=sum((Y-repmat(mean(Y), size(Y,1), 1)).^2);
model.R2=1-(RSS./TSS);


model.vip=vip;
model.TotVip=viptot;


if strcmp(opt.detailedoutput,'on')
    
    q=sum((Xp-T*P').^2,2);
    t2=diag((ns-1)*T/(T'*T)*T'); %tsquare
    t2lim=facts*(ns-1)*finv(0.95,facts,ns-facts)./(ns-facts);
    
    sres=sum(q/(ns-1));
    
    %Computation of qlim using J-M approx
    theta1 = sum(sres);
    theta2 = sum(sres.^2);
    theta3 = sum(sres.^3);
    if theta1==0;
        qlim = 0;
    else
        h0     = 1-2*theta1*theta3/3/(theta2.^2);
        if h0<0.001
            h0 = 0.001;
        end
        ca    = sqrt(2)*erfinv(2*0.95-1);
        h1    = ca*sqrt(2*theta2*h0.^2)/theta1;
        h2    = theta2*h0*(h0-1)/(theta1.^2);
        qlim = theta1*(1+h1+h2).^(1/h0);
    end
    
    model.tsq=t2;
    model.qres=q;
    model.tsqlim=t2lim;
    model.qlim=qlim;
    model.tsqr=t2/t2lim; 
    model.qr=q/qlim;
    
end


model.preprpars={mx sx; my sy};
model.options=opt;







%%%%%%%%%%%%%%%%
function [Mp,mm,sm]=prepfn(X,pret1)
%Preprocessing routine. Performs mean centering ('mean'), autoscaling
%('auto') or no pretreatment ('none')

nt=size(X,1);
switch pret1
    case 'none'
        Mp=X;
        mm=[];
        sm=[];
    case 'mean'
        mm=mean(X);
        sm=[];
        Mp=X-repmat(mm, nt, 1);
    case 'auto'
        mm=mean(X);
        sm=std(X);
        snzero=1:size(X,2);snzero(sm==0)=[];
        Mp=zeros(size(X));
        Mp(:,snzero)=(X(:,snzero)-repmat(mm(snzero), nt, 1))./repmat(sm(snzero),nt, 1);
    case 'pareto'
        mm=mean(X);
        sm=sqrt(std(X));
        snzero=1:size(X,2);snzero(sm==0)=[];
        Mp=zeros(size(X));
        Mp(:,snzero)=(X(:,snzero)-repmat(mm(snzero), nt, 1))./repmat(sm(snzero),nt, 1);
end

%%%%%%%%%%%%%%%%%%
function Mn=unprepfn(X,pret1,mm,sm)
%Preprocessing routine. Applies preprocessing to new matrices using the parameters
%calculated on the training set. Performs mean centering ('mean'), autoscaling
%('auto') or no pretreatment ('none')

nt=size(X,1);
switch pret1
    case 'none'
        Mn=X;
        
    case 'mean'
        Mn=X+repmat(mm, nt, 1);
    case {'auto', 'pareto'}
        Mn=(X.*repmat(sm,nt, 1))+repmat(mm, nt, 1);
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [T,P,U,R,W,Q,B, ssq]=pls2nip(X,Y, facts)

[ns,npx]=size(X);
[~,npy]=size(Y);
ssqx=zeros(facts,1);
ssqy=zeros(facts,1);

ss2x=sum(sum(X.^2));
ss2y=sum(sum(Y.^2));



E=X; F=Y; % Set E0=X and F0=Y
W=zeros(npx,facts); T=zeros(ns,facts); P=zeros(npx,facts); C=zeros(1,facts);
Q=zeros(npy,facts); U=T;   %Initialization of the matrices

for i=1:facts                         %Loop over the number of LVs
    [~,b]=max(std(F));      %Initial estimate of u as the Y(F) column with the largest variance
    u_old=F(:,b);
    crit=1;                 %initializing the convergence criterion
    while crit>10^(-9)      %loop for the computation of the ith component
        w=E'*u_old;
        w=w./sqrt(w'*w);
        t=E*w;
        q=F'*t; q=q./norm(q);
        u=F*q;
        crit=norm(u-u_old)./norm(u_old);  %check for convergence.
        u_old=u;
    end
    
    p=E'*t./(t'*t); c=u'*t./(t'*t);
    W(:,i)=w; T(:,i)=t; P(:,i)=p; C(i)= c; Q(:,i)=q; U(:,i)=u;
    E=E-t*p'; F=F-c*t*q';  %deflation of X and Y
    
    
    ssqx(i)=100*(sum(sum((T(:,i)*P(:,i)').^2))./ss2x);
    ssqy(i)=100*(sum(sum((U(:,i)*Q(:,i)').^2))./ss2y);
    
end


R=W/(P'*W); % R weights (allow direct calculation of T from X);

%Calculation of PLS regression coefficients
B=R*diag(C)*Q';

ssq={ssqx ssqy};


function [T,P,U,R,C,B,V, ssq]=pls2sim(Xs,Ys, facts)

[ns,nx]=size(Xs);
ny=size(Ys,2);

S=(Ys'*Xs)';

if ns>3*nx
    XtX=Xs'*Xs;
else
    XtX=[];
end			        % when XtX=[] and X is very "tall", the booster XtX is calculated

T=zeros(ns,facts);
U=T;						% initialization of variables
R=zeros(nx,facts);
P=R;
V=R;
C=zeros(ny,facts);
z=zeros(ny,1);

ssqx=zeros(facts,1);
ssqy=zeros(facts,1);

ss2x=sum(sum(Xs.^2));
ss2y=sum(sum(Ys.^2));

% if ns>nx
%     S0 = S;
% end

StS=S'*S;				    % SIMPLS algorithm
nm1=ns-1;

for a=1:facts
    StS=StS-z*z';
    [Q,LAMBDA]=eig(StS);
    [lambda,j]=max(diag(LAMBDA));
    q=Q(:,j(1));
    r=S*q;
    t=Xs*r;
    
    if isempty(XtX)
        p=(t'*Xs)';
    else
        p=XtX*r;
    end
    
    if ns>nx,
        d=sqrt(r'*p/nm1);
    else
        d=sqrt(t'*t/nm1);
    end
    
    v=p-V(:,1:max(1,a-1))*(p'*V(:,1:max(1,a-1)))';
    v=v/sqrt(v'*v);
    z=(v'*S)';
    S=S-v*z';
    V(:,a)=v;
    R(:,a)=r/d; 						    % X weights
    P(:,a)=p/(d*nm1); 						% X loadings
    T(:,a)=t/d;							    % X scores
    U(:,a)=Ys*q;							    % Y scores
    C(:,a)=q*(lambda(1)/(nm1*d)); 			% Y loadings
    
    ssqx(a)=100*(sum(sum((T(:,a)*P(:,a)').^2))./ss2x);
    ssqy(a)=100*(sum(sum((U(:,a)*C(:,a)').^2))./ss2y);
    
end
B=R*C';

ssq={ssqx ssqy};


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [vip, viptot]=myvip2(T,W,R,Q,B)

nx   = size(R,1);
ny   = size(Q,1);
vip=zeros(size(B));


TT = sum(T.^2,1);

if ~isempty(W)
    W_norm = (W*diag(1./sqrt(sum(W.^2,1))));  %normalized weights
else
    W_norm = (R*diag(1./sqrt(sum(R.^2,1))));  %normalized weights
end
SSfY=0;

for i = 1:ny;
    %calculate regression in terms of scores (T*b = y_hat)
    b  = R\B(:,i);
    
    %calculate weighted T^2
    SS = b.^2.*TT';
    
    SSfY=SSfY+SS;
    
    %VIP scores for this y
    vip(:,i) = sqrt(nx*W_norm.^2*SS./sum(SS));
end

%TOTAL VIP

sstot=sum(SSfY);

viptot = sqrt(nx*W_norm.^2*SSfY./sstot);
