function model = plsdacv(varargin )
% Calculates a PLS-DA model for the classification of samples, using cross-validation to
% select the optimal number of latent vectors. It requires
% a data matrix X and a second variable Y with class information which can be
% either a dummy binary-coded matrix: 
%           [ 1 0 0 0
%             1 0 0 0
%       Y=    0 1 0 0 
%             0 1 0 0
%              ......
%             0 0 0 1 ]
% 
% or a vector of class indices:      
%       Y=  [1 1 2 2 1 2 ... 3 1 4]'
% 
% I/O: model=plsdamod(X,Y, LV, options)
% where LV is the number of latent vectors and
% options is a structure array (if omitted default options will be used)
% with fields:
%   prepr: cell array 1x2 defining the desired preprocessing for X and Y blocks. 
%          options are 'mean', 'auto', 'pareto' and 'none'
%   algorithm: may be either 'nipals' or 'simpls'
%   detailedoutput: 'on'/'off'. If 'on' T2 and Q values are saved in the
%                    model
%   classcriterion: defines how classification is accomplished based on 
%                   the values of the predicted Y. So far it can be either 
%                   'maxY' (samples are assigned based on the highest component
%                   of the predicted Y) or 'lda' (samples are assigned
%                   after LDA on predicted Y or X scores.
%   ldax: defines the X to be used for LDA if 'lda' is selected as classcriterion.
%         It may be 'predy' (LDA is made on PLS predicted Y after removal
%         of one column) or 'scores' (LDA is made on PLS scores T.
%   prior: Defines which prior to use in LDA or bayesian classification. 
%          It can be 'uniform' or 'frequency' (prior based on the class
%          frequencies in the training set')
%   cv: struture defining the characteristics of cross-validation, with
%       fields cvtype and cvsegments. 
%       cvtype may be: 'loo', 'syst123', 'syst111', 'manual', 'random'
%       cvsegments defines the number of cross-validation splits. In case
%       manual cv is selected, cvsegments is a vector indicating the
%       cancelation group for each sample.
%
%
% Default options may be called by opt=plsdamod('options')
%
% All the results in cross-validation collected in the model.CrossVal field, 
% while the final model with the optimal number of LVs is saved in model.Model.



if nargin==1 &&strcmp(varargin{1}, 'options')
    model.prepr={'mean' 'mean'};
    model.algorithm='simpls';
    model.detailedoutput='on';
    model.classcriterion='maxY';
    model.ldax='predy';
    model.prior='uniform';
    model.cv.cvtype='syst123'; 
    model.cv.cvsegments=5;
    
    
    return
elseif nargin==3
    
    X=varargin{1};
    Y=varargin{2};
    LV=varargin{3};
    opt.prepr={'mean' 'mean'};
    opt.algorithm='simpls';
    opt.detailedoutput='on';
    opt.classcriterion='maxY';
    opt.ldax='predy';
    opt.prior='uniform';
    opt.cv.cvtype='syst123'; 
    opt.cv.cvsegments=5;
    
    
elseif nargin==4
    X=varargin{1};
    Y=varargin{2};
    LV=varargin{3};
    opt=varargin{4};
end


if isvector(Y)
    if size(Y,1)==1
        Y=Y';
    end
else
    [~,ycl]=max(Y,[],2);
    Y=ycl; 
end


cvtype=opt.cv.cvtype; 
segm=opt.cv.cvsegments; 
ns=size(X,1); 


if strcmp(cvtype, 'loo')
    cvtype='syst123'; 
    segm=ns; 
end

if strcmp(cvtype, 'manual')
    manseg=segm; 
    segm=max(manseg); 
end
if strcmp(cvtype, 'random')
    ridx=randperm(ns);
end

if strcmp(cvtype, 'syst111')
    nosam=ceil(ns/segm);
end

Ypredcv=zeros(ns,LV); 

for k=1:segm
    switch cvtype
        case 'syst123'
            t=k:segm:ns;
            m=1:ns; m(t)=[];
        case 'random'
            t=ridx(k:segm:ns);
            m=ridx; m(k:segm:ns)=[];
        case 'manual'
            t=find(manseg==k);
            m=1:ns; m(t)=[];
        case 'syst111'
            if k~=segm
                t=(k-1)*nosam+1:k*nosam;
                m=[1:(k-1)*nosam k*nosam+1:ns];
            else
                t=(k-1)*nosam+1:ns;
                m=1:(k-1)*nosam;
            end
    end
    
    Xtr=X(m,:); 
    Xts=X(t,:); 
    Ytr=Y(m,:); 
    Yts=Y(t,:); 
    
    
    for i=1:LV
        mi=plsdamod(Xtr, Ytr, i, opt);
        pi=plsdapred(Xts, Yts, mi);
        Ypredcv(t,i)=pi.classification.PredClass;
    end

end

model.CrossVal.PredClassCV=Ypredcv; 
model.CrossVal.TrueClass=Y;

cmatrix=cell(1,LV); 
ccrate=cell(1,LV);
cct=zeros(1,LV);

for i=1:LV
    cmatrix{i}=cmatrixcalc(Ypredcv(:,i), Y, max(Y));
    ccrate{i}=100*diag(cmatrix{i})./sum(cmatrix{i},2); 
    cct(i)=100*sum(diag(cmatrix{i}))/length(Y);
    
end

model.CrossVal.ConfusionMatrix=cmatrix; 
model.CrossVal.CorrClassRate=ccrate; 
model.CrossVal.TotalClassErr=cct; 

figure
plot(1:LV, 100-cct, 'o-r')
xlabel('Number of Latent Variables')
ylabel('% Classification Error')
title('Selection of the optimal number of LVs')

optLV=input('Please select the optimal number of LVs:   '); 



model.CrossVal.OptLV=optLV; 

mfin=plsdamod(X, Y, optLV, opt);

model.Model=mfin; 
model.options=opt; 


function cmatrix=cmatrixcalc(Ypred, Ytrue, nc)
ncts=max(Ytrue);

if ncts>nc
    cmatrix=zeros(ncts,nc);
else
    cmatrix=zeros(nc,nc);
end

for i=1:length(Ytrue)
    cmatrix(Ytrue(i), Ypred(i))=cmatrix(Ytrue(i), Ypred(i))+1;
end