function model = plsdamod(varargin )
% Calculates a PLS-DA model for the classification of samples. It requires
% a data matrix X and a second variable Y with class information which can be
% either a dummy binary-coded matrix: 
%           [ 1 0 0 0
%             1 0 0 0
%       Y=    0 1 0 0 
%             0 1 0 0
%              ......
%             0 0 0 1 ]
% 
% or a vector of class indices:      
%       Y=  [1 1 2 2 1 2 ... 3 1 4]'
% 
% I/O: model=plsdamod(X,Y, LV, options)
% where LV is the number of latent vectors and
% options is a structure array (if omitted default options will be used)
% with fields:
%   prepr: cell array 1x2 defining the desired preprocessing for X and Y blocks. 
%          options are 'mean', 'auto', 'pareto' and 'none'
%   algorithm: may be either 'nipals' or 'simpls'
%   detailedoutput: 'on'/'off'. If 'on' T2 and Q values are saved in the
%                    model
%   classcriterion: defines how classification is accomplished based on 
%                   the values of the predicted Y. So far it can be either 
%                   'maxY' (samples are assigned based on the highest component
%                   of the predicted Y) or 'lda' (samples are assigned
%                   after LDA on predicted Y or X scores.
%   ldax: defines the X to be used for LDA if 'lda' is selected as classcriterion.
%         It may be 'predy' (LDA is made on PLS predicted Y after removal
%         of one column) or 'scores' (LDA is made on PLS scores T.
%   prior: Defines which prior to use in LDA or bayesian classification. 
%          It can be 'uniform' or 'frequency' (prior based on the class
%          frequencies in the training set')
%
%
% Default options may be called by opt=plsdamod('options')
%
% All the results are collected in the model structure.




if nargin==1 &&strcmp(varargin{1}, 'options')
    model.prepr={'mean' 'mean'};
    model.algorithm='simpls';
    model.detailedoutput='on';
    model.classcriterion='maxY';
    model.ldax='predy';
    model.prior='uniform';
    
    return
elseif nargin==3
    
    X=varargin{1};
    Y=varargin{2};
    LV=varargin{3};
    opt.prepr={'mean' 'mean'};
    opt.algorithm='simpls';
    opt.detailedoutput='on';
    opt.classcriterion='maxY';
    opt.ldax='predy';
    opt.prior='uniform';
    
elseif nargin==4
    X=varargin{1};
    Y=varargin{2};
    LV=varargin{3};
    opt=varargin{4};
end

% Preparation of the class matrices and vectors

if isvector(Y)
    if size(Y,1)==1
        Y=Y';
    end
    ycl=Y;
    nclass=max(ycl);
    Y=zeros(length(ycl), nclass);
    for i=1:length(ycl)
        Y(i,ycl(i))=1;
    end
else
    [~,ycl]=max(Y,[],2);
    nclass=size(Y,2);
end


model=pls2mod(X,Y,LV, opt);

model.classification.TrueClass=ycl;
switch opt.classcriterion
    case 'maxY'
        [~,YPredClass]=max(model.preds,[],2);
        model.classification.PredClass=YPredClass;
    case 'lda'
        if strcmp(opt.ldax, 'scores')
            xx=model.scores{1};
        elseif strcmp(opt.ldax, 'predy')
            xx=model.preds;
            xx=xx(:,1:end-1);
        end
        
        mlda= ldaf(xx,xx,ycl, opt.prior);
        model.classification.PredClass=mlda.PredClass;
        model.classification.probability=mlda.PosteriorProb;
end


cmatrix=cmatrixcalc(model.classification.PredClass, ycl, nclass);
model.classification.ConfusionMatrix=cmatrix;
ccrate(1:size(cmatrix,2))=100*diag(cmatrix)./sum(cmatrix,2);
model.classification.CorrClassRate=ccrate;
model.classification.CorrClassTot=100*sum(diag(cmatrix))/length(ycl);

model.options=opt; 


function cmatrix=cmatrixcalc(Ypred, Ytrue, nc)
ncts=max(Ytrue);

if ncts>nc
    cmatrix=zeros(ncts,nc);
else
    cmatrix=zeros(nc,nc);
end

for i=1:length(Ytrue)
    cmatrix(Ytrue(i), Ypred(i))=cmatrix(Ytrue(i), Ypred(i))+1;
end
